/**
 * A class inspired by Chapter 3 from the O'Reilly book
 * "Database Programming with JDBC and Java, 2nd Edition
 * By George Reese ISBN: 1-56592-616-1.
 *
 * This servlet recreates the database for the comments
 *
 * In a real webapp, you would want to password protect this
 * functionality, and only let a trusted system manager do this
 *
 */

package edu.udel.pconrad.GuestBook;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.Date;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.Properties;
import java.util.Random;
import java.util.StringTokenizer;

import javax.servlet.ServletContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class GuestBookInitServlet extends HttpServlet {

    private Properties connectionProperties = new Properties();
    private Driver     driver               = null;
    private String     driverName           = null;
    private String     jdbcURL              = null;
    private String     dropDatabaseSQL    = null;
    private String     createDatabaseSQL    = null;
    private Random     random               = new Random();
    
    /**
     * Provides the servlet with the chance to get runtime configuration
     * values and initialize itself.  For a database servlet, you want
     * to grab the driver name, URL, and any connection information.
     * For this example, I assume a driver that requires a user name
     * and password.  For an example of more database independent
     * configuration, see Chapter 4.
     * @param cfg the servlet configuration information
     * @throws javax.servlet.ServletException could not load the specified
     * JDBC driver
     */

    public void init(ServletConfig cfg) throws ServletException {
        super.init(cfg);
        {
            String user, pw;

	    ServletContext ctx=getServletContext();
	    
            driverName = ctx.getInitParameter("gb.driver");
            jdbcURL = ctx.getInitParameter("gb.jdbcURL");
            user = ctx.getInitParameter("gb.user");
	    
	    dropDatabaseSQL = ctx.getInitParameter("gb.dropDatabaseSQL");
	    createDatabaseSQL = ctx.getInitParameter("gb.createDatabaseSQL");

            if( user != null ) {
                connectionProperties.put("user", user);
            }
            pw = ctx.getInitParameter("gb.pw");
            if( pw != null ) {
                connectionProperties.put("password", pw);
            }
            try {
                driver = (Driver)Class.forName(driverName).newInstance();
            }
            catch( Exception e ) {
                throw new ServletException("Unable to load driver: " +
                                           e.getMessage());
            }
        }
    }

    /**
     * Performs the HTTP POST.  This is where we create the database tables,
     * or reset them if they don't already exist.
     * @param req the servlet request information
     * @param res the servlet response information
     * @throws javax.servlet.ServletException an error occurred talking to
     * the database
     * @throws java.io.IOException a socket error occurred
     */

    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException {
        PrintWriter out = res.getWriter();
        Locale loc = getLocale(req);
        String name, email, comment;
        Connection conn = null;
        Exception err = null;
        int id = -1;
        String[] tmp;

	Statement stmt;

        try {
            conn = DriverManager.getConnection(jdbcURL, connectionProperties);

            stmt = conn.createStatement();
            // generate a new comment ID
            // more on ID generation in Chapter 4
            stmt.executeUpdate(dropDatabaseSQL);
            // closing the statement closes the result
            stmt.close();

            stmt = conn.createStatement();
            // generate a new comment ID
            // more on ID generation in Chapter 4
            stmt.executeUpdate(createDatabaseSQL);
            // closing the statement closes the result
            stmt.close();

        }
        catch( SQLException e ) {
            e.printStackTrace();
            err = e;
        }
        finally {
            if( conn != null ) {
                try { conn.close(); }
                catch( Exception e ) { }
            }
        }
        if( err != null ) {
	    out.println("Failed to drop and create database with sql:\"" 
		     + dropDatabaseSQL + "\n" + createDatabaseSQL + "\"");
            out.println("An error occurred: " + err.getMessage());
        }
        else {
            printResult(out, loc);
        }
    }

    /**
     * Find the desired locale from the HTTP header.
     * @param req the servlet request from which the header is read
     * @return the locale matching the first accepted language
     */
    private Locale getLocale(HttpServletRequest req) {
        String hdr = req.getHeader("Accept-Language");
        StringTokenizer toks;
        
        if( hdr == null ) {
            return Locale.getDefault();
        }
        toks = new StringTokenizer(hdr, ",");
        if( toks.hasMoreTokens() ) {
            String lang = toks.nextToken();
            int ind = lang.indexOf(';');
            Locale loc;
                
            if( ind != -1 ) {
                lang = lang.substring(0, ind);
            }
            lang = lang.trim();
            ind = lang.indexOf("-");
            if( ind == -1 ) {
                loc = new Locale(lang, "");
            }
            else {
                loc = new Locale(lang.substring(0, ind),
                                 lang.substring(ind+1));
            }
            return loc;
        }
        return Locale.getDefault();
    }
    
    public String getServletInfo() {
        return "Guest Book Servlet\nFrom Database Programming with JDBC " +
            "and Java";
    }

    private void printResult(PrintWriter out, Locale loc)
        throws IOException {
        out.println("<div>");
        out.println("<p>");
        out.println("The table was created successfully.");
        out.println("</p>");
        out.println("</div>");
    }

} // GuestBookInitServlet
